@@ -891,9 +891,9 @@ njs_regexp_builtin_exec(njs_vm_t *vm, njs_value_t *r, njs_value_t *s,
         offset = last_index;
 
     } else {
-        /* UTF-8 string. */
-        offset = njs_string_offset(string.start, string.start + string.size,
-                                   last_index) - string.start;
+        offset = njs_string_utf8_offset(string.start,
+                                        string.start + string.size, last_index)
+                 - string.start;
     }
 
     ret = njs_regexp_match(vm, &pattern->regex[type], string.start, offset,
@@ -1193,12 +1193,11 @@ njs_regexp_string_create(njs_vm_t *vm, njs_value_t *value, u_char *start,
 }
 
 
-static njs_int_t
+njs_int_t
 njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
     njs_uint_t nargs, njs_index_t unused)
 {
-    u_char             *p;
-    int64_t            n, last_index, ncaptures, pos, next_pos, size, length;
+    int64_t            n, last_index, ncaptures, pos, next_pos, length;
     njs_str_t          rep, m;
     njs_int_t          ret;
     njs_arr_t          results;
@@ -1289,6 +1288,10 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
         if (njs_fast_path(njs_is_fast_array(r) && njs_array_len(r) != 0)) {
             value = njs_array_start(r)[0];
 
+            if (!njs_is_valid(&value)) {
+                njs_set_undefined(&value);
+            }
+
         } else {
             ret = njs_value_property_i64(vm, r, 0, &value);
             if (njs_slow_path(ret == NJS_ERROR)) {
@@ -1357,7 +1360,8 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
 
         if ((size_t) length != s.size) {
             /* UTF-8 string. */
-            pos = njs_string_offset(s.start, s.start + s.size, pos) - s.start;
+            pos = njs_string_utf8_offset(s.start, s.start + s.size, pos)
+                  - s.start;
         }
 
         pos = njs_max(njs_min(pos, (int64_t) s.size), 0);
@@ -1455,23 +1459,12 @@ njs_regexp_prototype_symbol_replace(njs_vm_t *vm, njs_value_t *args,
         njs_chb_append(&chain, &s.start[next_pos], s.size - next_pos);
     }
 
-    size = njs_chb_size(&chain);
-    if (njs_slow_path(size < 0)) {
-        njs_memory_error(vm);
+    ret = njs_string_create_chb(vm, &vm->retval, &chain);
+    if (njs_slow_path(ret != NJS_OK)) {
         ret = NJS_ERROR;
         goto exception;
     }
 
-    length = njs_chb_utf8_length(&chain);
-
-    p = njs_string_alloc(vm, &vm->retval, size, length);
-    if (njs_slow_path(p == NULL)) {
-        ret = NJS_ERROR;
-        goto exception;
-    }
-
-    njs_chb_join_to(&chain, p);
-
     ret = NJS_OK;
 
 exception:
@@ -1651,8 +1644,8 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
         }
 
         if (utf8 == NJS_STRING_UTF8) {
-            start = njs_string_offset(s.start, s.start + s.size, p);
-            end = njs_string_offset(s.start, s.start + s.size, q);
+            start = njs_string_utf8_offset(s.start, s.start + s.size, p);
+            end = njs_string_utf8_offset(s.start, s.start + s.size, q);
 
         } else {
             start = &s.start[p];
@@ -1699,7 +1692,8 @@ njs_regexp_prototype_symbol_split(njs_vm_t *vm, njs_value_t *args,
     end = &s.start[s.size];
 
     if (utf8 == NJS_STRING_UTF8) {
-        start = (p < length) ? njs_string_offset(s.start, s.start + s.size, p)
+        start = (p < length) ? njs_string_utf8_offset(s.start, s.start + s.size,
+                                                      p)
                              : end;
 
     } else {
