@@ -145,6 +145,7 @@ njs_vm_destroy(njs_vm_t *vm)
 njs_int_t
 njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
 {
+    size_t              global_items;
     njs_int_t           ret;
     njs_str_t           ast;
     njs_chb_t           chain;
@@ -156,6 +157,8 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
 
     vm->codes = NULL;
 
+    global_items = (vm->global_scope != NULL) ? vm->global_scope->items : 0;
+
     ret = njs_parser_init(vm, &parser, vm->global_scope, &vm->options.file,
                           *start, end, 0);
     if (njs_slow_path(ret != NJS_OK)) {
@@ -202,9 +205,7 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
         return NJS_ERROR;
     }
 
-    vm->global_scope = scope;
-
-    if (scope->items > vm->global_items) {
+    if (scope->items > global_items) {
         global = vm->levels[NJS_LEVEL_GLOBAL];
 
         new = njs_scope_make(vm, scope->items);
@@ -215,8 +216,8 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
         vm->levels[NJS_LEVEL_GLOBAL] = new;
 
         if (global != NULL) {
-            while (vm->global_items != 0) {
-                vm->global_items--;
+            while (global_items != 0) {
+                global_items--;
 
                 *new++ = *global++;
             }
@@ -227,8 +228,7 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
     njs_scope_value_set(vm, njs_scope_global_this_index(), &vm->global_value);
 
     vm->start = generator.code_start;
-    vm->variables_hash = &scope->variables;
-    vm->global_items = scope->items;
+    vm->global_scope = scope;
 
     if (vm->options.disassemble) {
         njs_disassembler(vm);
@@ -238,6 +238,13 @@ njs_vm_compile(njs_vm_t *vm, u_char **start, u_char *end)
 }
 
 
+njs_mod_t *
+njs_vm_add_module(njs_vm_t *vm, njs_str_t *name, njs_value_t *value)
+{
+    return njs_module_add(vm, name, value);
+}
+
+
 njs_mod_t *
 njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
     u_char *end)
@@ -256,7 +263,7 @@ njs_vm_compile_module(njs_vm_t *vm, njs_str_t *name, u_char **start,
         return module;
     }
 
-    module = njs_module_add(vm, name);
+    module = njs_module_add(vm, name, NULL);
     if (njs_slow_path(module == NULL)) {
         return NULL;
     }
@@ -344,7 +351,7 @@ njs_vm_clone(njs_vm_t *vm, njs_external_ptr_t external)
         goto fail;
     }
 
-    global = njs_scope_make(nvm, nvm->global_items);
+    global = njs_scope_make(nvm, nvm->global_scope->items);
     if (njs_slow_path(global == NULL)) {
         goto fail;
     }
@@ -663,6 +670,13 @@ njs_vm_memory_pool(njs_vm_t *vm)
 }
 
 
+njs_external_ptr_t
+njs_vm_external_ptr(njs_vm_t *vm)
+{
+    return vm->external;
+}
+
+
 uintptr_t
 njs_vm_meta(njs_vm_t *vm, njs_uint_t index)
 {
@@ -818,6 +832,22 @@ njs_vm_value_string_alloc(njs_vm_t *vm, njs_value_t *value, uint32_t size)
 }
 
 
+njs_int_t
+njs_vm_value_string_create(njs_vm_t *vm, njs_value_t *value,
+    const u_char *start, uint32_t size)
+{
+    return njs_string_create(vm, value, (const char *) start, size);
+}
+
+
+njs_int_t
+njs_vm_value_string_create_chb(njs_vm_t *vm, njs_value_t *value,
+    njs_chb_t *chain)
+{
+    return njs_string_create_chb(vm, value, chain);
+}
+
+
 njs_function_t *
 njs_vm_function(njs_vm_t *vm, const njs_str_t *path)
 {
@@ -892,7 +922,7 @@ njs_vm_logger(njs_vm_t *vm, njs_log_level_t level, const char *fmt, ...)
     u_char        *p;
     va_list       args;
     njs_logger_t  logger;
-    u_char        buf[NJS_MAX_ERROR_STR];
+    u_char        buf[32768];
 
     if (vm->options.ops == NULL) {
         return;
@@ -1227,6 +1257,46 @@ njs_vm_value_to_string(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 }
 
 
+/*
+ * If string value is null-terminated the corresponding C string
+ * is returned as is, otherwise the new copy is allocated with
+ * the terminating zero byte.
+ */
+const char *
+njs_vm_value_to_c_string(njs_vm_t *vm, njs_value_t *value)
+{
+    u_char  *p, *data, *start;
+    size_t  size;
+
+    njs_assert(njs_is_string(value));
+
+    if (value->short_string.size != NJS_STRING_LONG) {
+        start = value->short_string.start;
+        size = value->short_string.size;
+
+        if (size < NJS_STRING_SHORT) {
+            start[size] = '\0';
+            return (const char *) start;
+        }
+
+    } else {
+        start = value->long_string.data->start;
+        size = value->long_string.size;
+    }
+
+    data = njs_mp_alloc(vm->mem_pool, size + njs_length("\0"));
+    if (njs_slow_path(data == NULL)) {
+        njs_memory_error(vm);
+        return NULL;
+    }
+
+    p = njs_cpymem(data, start, size);
+    *p++ = '\0';
+
+    return (const char *) data;
+}
+
+
 njs_int_t
 njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 {
@@ -1300,6 +1370,13 @@ njs_vm_value_to_bytes(njs_vm_t *vm, njs_str_t *dst, njs_value_t *src)
 }
 
 
+njs_int_t
+njs_vm_string_compare(const njs_value_t *v1, const njs_value_t *v2)
+{
+    return njs_string_cmp(v1, v2);
+}
+
+
 njs_int_t
 njs_vm_value_string_copy(njs_vm_t *vm, njs_str_t *retval,
     njs_value_t *value, uintptr_t *next)
