@@ -195,7 +195,7 @@ njs_vmcode_interpreter(njs_vm_t *vm, u_char *pc, void *promise_cap,
         NJS_GOTO_ROW(NJS_VMCODE_ADDITION),
         NJS_GOTO_ROW(NJS_VMCODE_EQUAL),
         NJS_GOTO_ROW(NJS_VMCODE_NOT_EQUAL),
-        NJS_GOTO_ROW(NJS_VMCODE_SUBSTRACTION),
+        NJS_GOTO_ROW(NJS_VMCODE_SUBTRACTION),
         NJS_GOTO_ROW(NJS_VMCODE_MULTIPLICATION),
         NJS_GOTO_ROW(NJS_VMCODE_EXPONENTIATION),
         NJS_GOTO_ROW(NJS_VMCODE_DIVISION),
@@ -712,7 +712,7 @@ NEXT_LBL;
         njs_vmcode_operand(vm, vmcode->operand1, retval);                     \
         pc += sizeof(njs_vmcode_3addr_t)
 
-    CASE (NJS_VMCODE_SUBSTRACTION):
+    CASE (NJS_VMCODE_SUBTRACTION):
         njs_vmcode_debug_opcode();
 
         njs_vmcode_operand(vm, vmcode->operand3, value2);
@@ -1355,8 +1355,8 @@ NEXT_LBL;
     CASE (NJS_VMCODE_IF_TRUE_JUMP):
         njs_vmcode_debug_opcode();
 
-        value2 = (njs_value_t *) vmcode->operand1;
         njs_vmcode_operand(vm, vmcode->operand2, value1);
+        value2 = (njs_value_t *) vmcode->operand1;
 
         ret = njs_is_true(value1);
 
@@ -1368,8 +1368,8 @@ NEXT_LBL;
     CASE (NJS_VMCODE_IF_FALSE_JUMP):
         njs_vmcode_debug_opcode();
 
-        value2 = (njs_value_t *) vmcode->operand1;
         njs_vmcode_operand(vm, vmcode->operand2, value1);
+        value2 = (njs_value_t *) vmcode->operand1;
 
         ret = njs_is_true(value1);
 
@@ -1824,7 +1824,12 @@ error:
 
     if (njs_is_error(&vm->retval)) {
         vm->active_frame->native.pc = pc;
-        (void) njs_error_stack_attach(vm, &vm->retval);
+
+        /* TODO: get rid of copying. */
+
+        njs_value_assign(&dst, &vm->retval);
+        (void) njs_error_stack_attach(vm, &dst);
+        njs_value_assign(&vm->retval, &dst);
     }
 
     for ( ;; ) {
@@ -2507,7 +2512,7 @@ again:
     /* If "hv" is a string then "lv" can be a numeric or symbol. */
     if (njs_is_string(hv)) {
         return !njs_is_symbol(lv)
-            && (njs_number(lv) == njs_string_to_number(hv, 0));
+            && (njs_number(lv) == njs_string_to_number(hv));
     }
 
     /* "hv" is an object and "lv" is either a string or a symbol or a numeric. */
@@ -2544,11 +2549,11 @@ njs_primitive_values_compare(njs_vm_t *vm, njs_value_t *val1, njs_value_t *val2)
             num2 = njs_number(val2);
 
         } else {
-            num2 = njs_string_to_number(val2, 0);
+            num2 = njs_string_to_number(val2);
         }
 
     } else if (njs_is_numeric(val2)) {
-        num1 = njs_string_to_number(val1, 0);
+        num1 = njs_string_to_number(val1);
         num2 = njs_number(val2);
 
     } else {
